# -*- coding: utf-8 -*-

from django.conf import settings
if 'localeurl' in settings.INSTALLED_APPS:
    from localeurl.models import patch_reverse
    patch_reverse()

from bs4 import BeautifulSoup
from datetime import datetime, timedelta

from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core import mail
from django.core import management
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.utils import timezone

from model_mommy import mommy

from coop_cms.models import Newsletter, NewsletterItem, PieceOfHtml, NewsletterSending
from coop_cms.settings import is_localized, get_article_class
from coop_cms.tests import BaseTestCase, UserBaseTestCase
from coop_cms.utils import make_links_absolute


class NewsletterTest(UserBaseTestCase):

    def test_create_article_for_newsletter(self):
        Article = get_article_class()
        ct = ContentType.objects.get_for_model(Article)
        
        art = mommy.make(Article, in_newsletter=True)
        
        self.assertEqual(1, NewsletterItem.objects.count())
        item = NewsletterItem.objects.get(content_type=ct, object_id=art.id)
        self.assertEqual(item.content_object, art)
        
        art.delete()
        self.assertEqual(0, NewsletterItem.objects.count())

    def test_create_article_not_for_newsletter(self):
        Article = get_article_class()
        ct = ContentType.objects.get_for_model(Article)
        
        art = mommy.make(Article, in_newsletter=False)
        self.assertEqual(0, NewsletterItem.objects.count())
        
        art.delete()
        self.assertEqual(0, NewsletterItem.objects.count())

    def test_create_article_commands(self):
        Article = get_article_class()
        ct = ContentType.objects.get_for_model(Article)
        art1 = mommy.make(Article, in_newsletter=True)
        art2 = mommy.make(Article, in_newsletter=True)
        art3 = mommy.make(Article, in_newsletter=False)
        self.assertEqual(2, NewsletterItem.objects.count())
        NewsletterItem.objects.all().delete()
        self.assertEqual(0, NewsletterItem.objects.count())
        management.call_command('create_newsletter_items', verbosity=0, interactive=False)
        self.assertEqual(2, NewsletterItem.objects.count())
        item1 = NewsletterItem.objects.get(content_type=ct, object_id=art1.id)
        item2 = NewsletterItem.objects.get(content_type=ct, object_id=art2.id)

    def test_view_newsletter(self):
        article_class = get_article_class()
        ct = ContentType.objects.get_for_model(article_class)
        
        art1 = mommy.make(article_class, title="Art 1", in_newsletter=True)
        art2 = mommy.make(article_class, title="Art 2", in_newsletter=True)
        art3 = mommy.make(article_class, title="Art 3", in_newsletter=True)
        
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=True
        )
        newsletter.items.add(NewsletterItem.objects.get(content_type=ct, object_id=art1.id))
        newsletter.items.add(NewsletterItem.objects.get(content_type=ct, object_id=art2.id))
        newsletter.save()
        
        url = reverse('coop_cms_view_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
        self.assertContains(response, newsletter.content)
        self.assertContains(response, art1.title)
        self.assertContains(response, art2.title)
        self.assertNotContains(response, art3.title)

    def test_view_newsletter_not_public(self):

        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
        )

        url = reverse('coop_cms_view_newsletter', args=[newsletter.id])
        response = self.client.get(url)

        self.assertEqual(403, response.status_code)
        
    def test_edit_newsletter_is_public(self):
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=True
        )

        self._log_as_editor()
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
        data = {'content': 'A better intro'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        self.assertNotContains(response, newsletter.content)
        self.assertContains(response, data['content'])

    def test_edit_newsletter_not_public(self):
        newsletter = mommy.make(
            Newsletter,
            content="a little intro for this newsletter",
            template="test/newsletter_blue.html",
            is_public=False
        )

        self._log_as_editor()
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

        data = {'content': 'A better intro'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, newsletter.content)
        self.assertContains(response, data['content'])

    def test_edit_newsletter_anonymous(self):
        original_data = {
            'content': "a little intro for this newsletter",
            'template': "test/newsletter_blue.html"
        }
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
        
        response = self.client.post(url, data={'content': ':OP'})
        self.assertEqual(403, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.content, original_data['content'])
        
    def test_edit_newsletter_no_articles(self):
        self._log_as_editor()
        original_data = {'content': "a little intro for this newsletter",
            'template': "test/newsletter_blue.html"}
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_edit_newsletter', args=[newsletter.id])
        
        data = {'content': ':OP'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, data['content'])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, data['content'])
        
    def test_newsletter_templates(self):
        
        Article = get_article_class()
        ct = ContentType.objects.get_for_model(Article)
        
        art1 = mommy.make(Article, title="Art 1", in_newsletter=True)
        poh = mommy.make(PieceOfHtml, div_id="newsletter_header", content="HELLO!!!")
        
        newsletter = mommy.make(Newsletter, content="a little intro for this newsletter",
            template="test/newsletter_blue.html")
        newsletter.items.add(NewsletterItem.objects.get(content_type=ct, object_id=art1.id))
        newsletter.save()
        
        self._log_as_editor()
        
        view_names = ['coop_cms_view_newsletter', 'coop_cms_edit_newsletter']
        for view_name in view_names:
            url = reverse(view_name, args=[newsletter.id])
            response = self.client.get(url)
            self.assertEqual(200, response.status_code)
            
            self.assertContains(response, newsletter.content)
            self.assertContains(response, art1.title)
            self.assertContains(response, "background: blue;")
            self.assertNotContains(response, poh.content)
        
        newsletter.template = "test/newsletter_red.html"
        newsletter.save()
        
        for view_name in view_names:
            url = reverse(view_name, args=[newsletter.id])
            response = self.client.get(url)
            
            self.assertEqual(200, response.status_code)
            
            self.assertContains(response, newsletter.content)
            self.assertContains(response, art1.title)
            self.assertContains(response, "background: red;")
            self.assertContains(response, poh.content)
            
    def test_change_newsletter_templates(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        self._log_as_editor()
        
        newsletter = mommy.make(Newsletter, template='test/newsletter_blue.html')
        
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        for tpl, name in settings.COOP_CMS_NEWSLETTER_TEMPLATES:
            self.assertContains(response, tpl)
            self.assertContains(response, name)
            
        data={'template': 'test/newsletter_red.html'}
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, reverse('coop_cms_edit_newsletter', args=[newsletter.id]))
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, data['template'])
        
    def test_change_newsletter_templates_anonymous(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        original_data={'template': 'test/newsletter_blue.html'}
        newsletter = mommy.make(Newsletter, **original_data)
        
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        
        data={'template': 'test/newsletter_red.html'}
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, original_data['template'])
        
    def test_change_newsletter_unknow_template(self):
        settings.COOP_CMS_NEWSLETTER_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        original_data={'template': 'test/newsletter_blue.html'}
        newsletter = mommy.make(Newsletter, **original_data)
        
        self._log_as_editor()
        url = reverse('coop_cms_change_newsletter_template', args=[newsletter.id])
        data={'template': 'test/newsletter_yellow.html'}
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        
        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.template, original_data['template'])
        
    def test_send_test_newsletter(self, template='test/newsletter_blue.html', extra_checker=None):
        settings.COOP_CMS_FROM_EMAIL = 'contact@toto.fr'
        settings.COOP_CMS_TEST_EMAILS = ('toto@toto.fr', 'titi@toto.fr')
        #settings.COOP_CMS_SITE_PREFIX = 'http://toto.fr'
        settings.SITE_ID = 1
        site = Site.objects.get(id=settings.SITE_ID)
        site.domain = 'toto.fr'
        site.save()
        
        rel_content = u'''
            <h1>Title</h1><a href="{0}/toto/"><img src="{0}/toto.jpg"></a><br /><img src="{0}/toto.jpg">
            <div><a href="http://www.google.fr">Google</a></div>
        '''
        original_data = {
            'template': template,
            'subject': 'test email',
            'content': rel_content.format("")
        }
        newsletter = mommy.make(Newsletter, **original_data)
        
        self._log_as_editor()
        url = reverse('coop_cms_test_newsletter', args=[newsletter.id])
        response = self.client.post(url, data={})
        self.assertEqual(200, response.status_code)
        
        self.assertEqual([[e] for e in settings.COOP_CMS_TEST_EMAILS], [e.to for e in mail.outbox])
        for e in mail.outbox:
            self.assertEqual(e.from_email, settings.COOP_CMS_FROM_EMAIL)
            self.assertEqual(e.subject, newsletter.subject)
            self.assertTrue(e.body.find('Title')>=0)
            self.assertTrue(e.body.find('Google')>=0)
            self.assertTrue(e.alternatives[0][1], "text/html")
            self.assertTrue(e.alternatives[0][0].find('Title')>=0)
            self.assertTrue(e.alternatives[0][0].find('Google')>=0)
            site_prefix = "http://"+site.domain
            self.assertTrue(e.alternatives[0][0].find(site_prefix)>=0)
            if extra_checker:
                extra_checker(e)
        
    def test_schedule_newsletter_sending(self):
        newsletter = mommy.make(Newsletter)
        
        self._log_as_editor()
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        sch_dt = "2030-12-12 12:00:00"
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertContains(response, '$.colorbox.close()')
        self.assertEqual(1, NewsletterSending.objects.count())
        self.assertEqual(newsletter, NewsletterSending.objects.all()[0].newsletter)
        self.assertEqual(2030, NewsletterSending.objects.all()[0].scheduling_dt.year)
        
    def test_schedule_newsletter_sending_invalid_value(self):
        newsletter = mommy.make(Newsletter)
        
        self._log_as_editor()
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
        sch_dt = ''
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
        
        sch_dt = 'toto'
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
        
        sch_dt = "2005-12-12 12:00:00"
        response = self.client.post(url, data={'scheduling_dt': sch_dt})
        self.assertEqual(200, response.status_code)
        self.assertEqual(0, NewsletterSending.objects.count())
    
    def test_schedule_anonymous(self):
        newsletter = mommy.make(Newsletter)
        
        login_url = reverse('django.contrib.auth.views.login')
        url = reverse('coop_cms_schedule_newsletter_sending', args=[newsletter.id])
        
        response = self.client.get(url, follow=False)
        redirect_url = response['Location']
        if is_localized():
            login_url = login_url[:2]
            self.assertTrue(redirect_url.find(login_url)>0)
        else:
            self.assertTrue(redirect_url.find(login_url)>0)
        
        sch_dt = timezone.now()+timedelta(1)
        response = self.client.post(url, data={'sending_dt': sch_dt})
        redirect_url = response['Location']
        self.assertTrue(redirect_url.find(login_url)>0)
    
    def test_send_newsletter(self):
        
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        self.assertEqual(newsletter.is_public, False)
        sch_dt = timezone.now() - timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)
        
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        
        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertNotEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, ['toto@toto.fr'])
        self.assertEqual(email.subject, newsletter_data['subject'])
        self.assertTrue(email.body.find('Hello guys') >= 0)
        self.assertTrue(email.alternatives[0][1], "text/html")
        self.assertTrue(email.alternatives[0][0].find('Hello guys') >= 0)

        newsletter = Newsletter.objects.get(id=newsletter.id)
        self.assertEqual(newsletter.is_public, True)
        
        #check whet happens if command is called again
        mail.outbox = []
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        self.assertEqual(len(mail.outbox), 0)
        
        
    def test_send_newsletter_several(self):
        
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        
        sch_dt = timezone.now() - timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)
        
        addresses = ';'.join(['toto@toto.fr']*5)
        management.call_command('send_newsletter', addresses, verbosity=0, interactive=False)
        
        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertNotEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 5)
        for email in mail.outbox:
            self.assertEqual(email.to, ['toto@toto.fr'])
            self.assertEqual(email.subject, newsletter_data['subject'])
            self.assertTrue(email.body.find('Hello guys')>=0)
            self.assertTrue(email.alternatives[0][1], "text/html")
            self.assertTrue(email.alternatives[0][0].find('Hello guys')>=0)
        
        #check whet happens if command is called again
        mail.outbox = []
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        self.assertEqual(len(mail.outbox), 0)

    def test_send_newsletter_not_yet(self):
        
        newsletter_data = {
            'subject': 'This is the subject',
            'content': '<h2>Hello guys!</h2><p>Visit <a href="http://toto.fr">us</a></p>',
            'template': 'test/newsletter_blue.html',
        }
        newsletter = mommy.make(Newsletter, **newsletter_data)
        
        sch_dt = timezone.now() + timedelta(1)
        sending = mommy.make(NewsletterSending, newsletter=newsletter, scheduling_dt= sch_dt, sending_dt= None)
        
        management.call_command('send_newsletter', 'toto@toto.fr', verbosity=0, interactive=False)
        
        sending = NewsletterSending.objects.get(id=sending.id)
        self.assertEqual(sending.sending_dt, None)
        
        self.assertEqual(len(mail.outbox), 0)


class AbsUrlTest(UserBaseTestCase):
    
    def setUp(self):
        super(AbsUrlTest, self).setUp()
        settings.SITE_ID = 1
        self.site = Site.objects.get(id=settings.SITE_ID)
        self.site.domain = "toto.fr"
        self.site.save()
        self.site_prefix = "http://"+self.site.domain
        self.newsletter = mommy.make(Newsletter, site=self.site)
        #settings.COOP_CMS_SITE_PREFIX = self.site_prefix
    
    def test_href(self):
        settings.COOP_CMS_SITE_PREFIX = self.site_prefix
        test_html = '<a href="%s/toto">This is a link</a>'
        rel_html = test_html % ""
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html))
    
    def test_href(self):
        test_html = '<a href="%s/toto">This is a link</a>'
        rel_html = test_html % ""
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
        
    def test_src(self):
        test_html = '<h1>My image</h1><img src="%s/toto">'
        rel_html = test_html % ""
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
        
    def test_relative_path(self):
        test_html = '<h1>My image</h1><img src="%s/toto">'
        rel_html = test_html % "../../.."
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
    
    def test_src_and_img(self):
        test_html = '<h1>My image</h1><a href="{0}/a1">This is a link</a><img src="{0}/toto"/><img src="{0}/titi"/><a href="{0}/a2">This is another link</a>'
        rel_html = test_html.format("")
        html = test_html.format(self.site_prefix)
        abs_html = BeautifulSoup(html).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
        
    def test_href_rel_and_abs(self):
        test_html = '<a href="%s/toto">This is a link</a><a href="http://www.apidev.fr">another</a>'
        rel_html = test_html % ""
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
        
    def test_style_in_between(self):
        test_html = u'<img style="margin: 0; width: 700px;" src="%s/media/img/newsletter_header.png" alt="Logo">'
        rel_html = test_html % ""
        abs_html = BeautifulSoup(test_html % self.site_prefix).prettify()
        self.assertEqual(abs_html, make_links_absolute(rel_html, self.newsletter))
        
    def test_missing_attr(self):
        test_html = u'<img alt="Logo" /><a name="aa">link</a>'
        abs_html = BeautifulSoup(test_html).prettify()
        self.assertEqual(abs_html, make_links_absolute(test_html, self.newsletter))


class NewsletterFriendlyTemplateTagsTest(BaseTestCase):
    
    template_content = """
        {{% load coop_utils %}}
        {{% nlf_css {0} %}}
            <a>One</a>
            <a>Two</a>
            <a>Three</a>
            <img />
            <table><tr><td></td><td></td></table>
            <table class="this-one"><tr><td></td><td></td></table>
        {{% end_nlf_css %}}
    """
    
    def test_email_mode_is_inline(self):
        template = self.template_content.format('a="color: red;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(3, html.count(u'<a style="color: red;">'))
        
    def test_edit_mode_is_in_style(self):
        template = self.template_content.format('a="color: red;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': False}))
        self.assertEqual(1, html.count(u'<style>'))
        self.assertEqual(1, html.count(u'a { color: red; }'))
        self.assertEqual(0, html.count(u'<a style="color: red;">'))
        
    def test_several_args_email_mode_is_inline(self):
        template = self.template_content.format('a="color: red; background: blue;" td="border: none;" img="width: 100px;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(3, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(1, html.count(u'<img style="width: 100px;"/>'))
        self.assertEqual(4, html.count(u'<td style="border: none;">'))
        
    def test_several_args_edit_mode_is_in_style(self):
        template = self.template_content.format('a="color: red; background: blue;" td="border: none;" img="width: 100px;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': False}))
        self.assertEqual(1, html.count(u'<style>'))
        self.assertEqual(1, html.count(u'a { color: red; background: blue; }'))
        self.assertEqual(1, html.count(u'img { width: 100px; }'))
        self.assertEqual(1, html.count(u'td { border: none; }'))
        self.assertEqual(0, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(0, html.count(u'<img style="width: 100px;">'))
        self.assertEqual(0, html.count(u'<td style="border: none;">'))
        
    def test_class_selector_email_mode_is_inline(self):
        template = self.template_content.format('"table.this-one td"="border: none;"')
        tpl = Template(template)
        html = tpl.render(Context({'by_email': True}))
        self.assertEqual(0, html.count(u'<style>'))
        self.assertEqual(0, html.count(u'<a style="color: red; background: blue;">'))
        self.assertEqual(0, html.count(u'<img style="width: 100px;"/>'))
        self.assertEqual(2, html.count(u'<td style="border: none;">'))





    




    




# -*- coding: utf-8 -*-

from django.conf import settings
if 'localeurl' in settings.INSTALLED_APPS:
    from localeurl.models import patch_reverse
    patch_reverse()

from bs4 import BeautifulSoup
from unittest import skipIf


from django.contrib.auth.models import AnonymousUser
from django.middleware.csrf import REASON_NO_REFERER, REASON_NO_CSRF_COOKIE
from django.test.client import RequestFactory

from coop_cms.tests import BaseTestCase
from coop_cms.views import csrf_failure


@skipIf(getattr(settings, 'COOP_CMS_DO_NOT_INSTALL_CSRF_FAILURE_VIEW', False), "coo_cms csrf failure disabled")
class CsrfFailureTest(BaseTestCase):
    
    def test_view_reason_cookie(self):
        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        
        response = csrf_failure(request, REASON_NO_CSRF_COOKIE)
        
        self.assertEqual(403, response.status_code)
        soup = BeautifulSoup(response.content)
        
        self.assertEqual(1, len(soup.select('.cookies-error')))
        self.assertEqual(0, len(soup.select('.referer-error')))
        self.assertEqual(0, len(soup.select('.unknown-error')))
        
    
    def test_view_reason_referer(self):
        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        
        response = csrf_failure(request, REASON_NO_REFERER)
        
        self.assertEqual(403, response.status_code)
        soup = BeautifulSoup(response.content)
        
        self.assertEqual(0, len(soup.select('.cookies-error')))
        self.assertEqual(1, len(soup.select('.referer-error')))
        self.assertEqual(0, len(soup.select('.unknown-error')))
    
    def test_view_reason_unknown(self):
        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        
        response = csrf_failure(request, "?")
        
        self.assertEqual(403, response.status_code)
        soup = BeautifulSoup(response.content)
        
        self.assertEqual(0, len(soup.select('.cookies-error')))
        self.assertEqual(0, len(soup.select('.referer-error')))
        self.assertEqual(1, len(soup.select('.unknown-error')))



# -*- coding: utf-8 -*-

from django.conf import settings
if 'localeurl' in settings.INSTALLED_APPS:
    from localeurl.models import patch_reverse
    patch_reverse()

from bs4 import BeautifulSoup

from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.template import Template, Context

from model_mommy import mommy
from coop_cms.models import BaseArticle, Fragment, FragmentType, FragmentFilter
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseTestCase

class FragmentsTest(BaseTestCase):
    
    editable_field_tpl = '<div class="djaloha-editable" id="djaloha_djaloha__coop_cms__Fragment__id__{0}__content">' + \
        '{1}</div>\n<input type="hidden" id="djaloha_djaloha__coop_cms__Fragment__id__{0}__content_hidden" ' + \
        'name="djaloha__coop_cms__Fragment__id__{0}__content" value="{1}">'
    
    def setUp(self):
        super(FragmentsTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/article_with_fragments.html', 'Article with fragments'),
            ('test/article_with_fragments_extra_id.html', 'Article with fragments extra id'),
            ('test/article_with_fragments_template.html', 'Article with fragments template'),
        )
        
    def tearDown(self):
        super(FragmentsTest, self).tearDown()
        #restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates

    def test_fragment_position(self):
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        
        f1 = mommy.make(Fragment, type=ft1)
        f2 = mommy.make(Fragment, type=ft1)
        f3 = mommy.make(Fragment, type=ft1)
        f4 = mommy.make(Fragment, type=ft1)
        
        g1 = mommy.make(Fragment, type=ft2)
        g2 = mommy.make(Fragment, type=ft2)
        g3 = mommy.make(Fragment, type=ft2)
        
        f5 = mommy.make(Fragment, type=ft1)
        
        for idx, elt in enumerate([f1, f2, f3, f4, f5]):
            self.assertEqual(idx+1, elt.position)
        
        for idx, elt in enumerate([g1, g2, g3]):
            self.assertEqual(idx+1, elt.position)
            
    def test_fragment_position_extra_id(self):
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        ff1 = mommy.make(FragmentFilter)
        ff2 = mommy.make(FragmentFilter)
        
        f1 = mommy.make(Fragment, type=ft1, filter=ff1)
        f2 = mommy.make(Fragment, type=ft1, filter=ff1)
        f3 = mommy.make(Fragment, type=ft1, filter=ff1)
        
        f4 = mommy.make(Fragment, type=ft1, filter=ff2)
        
        g1 = mommy.make(Fragment, type=ft2, filter=ff1)
        g2 = mommy.make(Fragment, type=ft2, filter=ff2)
        g3 = mommy.make(Fragment, type=ft2, filter=ff2)
        
        f5 = mommy.make(Fragment, type=ft1, filter=ff1)
        
        f6 = mommy.make(Fragment, type=ft1)
        f7 = mommy.make(Fragment, type=ft1)
        
        for idx, elt in enumerate([f1, f2, f3, f5]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([f4]):
            self.assertEqual(idx+1, elt.position)
        
        for idx, elt in enumerate([g1]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([g2, g3]):
            self.assertEqual(idx+1, elt.position)
            
        for idx, elt in enumerate([f6, f7]):
            self.assertEqual(idx+1, elt.position)
            
    def test_fragment_position_update(self):
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        
        f1 = mommy.make(Fragment, type=ft1)
        f2 = mommy.make(Fragment, type=ft1)
        f3 = mommy.make(Fragment, type=ft1)
        
        f1.save()
        f2.save()
        f3.save()
        
        for idx, elt in enumerate([f1, f2, f3]):
            self.assertEqual(idx+1, elt.position)
            
    def test_view_fragments(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty")
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh")
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f1, f2, f3]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_fragments_extra_id(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        ff1 = mommy.make(FragmentFilter, extra_id="1")
        ff2 = mommy.make(FragmentFilter, extra_id="2")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty", filter=ff1)
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh", filter=ff1)
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn", filter=ff2)
        f4 = mommy.make(Fragment, type=ft1, content="Zsxdrg", filter=None)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name x %}')
        html = tpl.render(Context({"ft_name": ft_name, "x": 1}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f1, f2]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
        soup = BeautifulSoup(html)
        ft_tags = soup.select(".coop-fragment-type")
        self.assertEqual(len(ft_tags), 1)
        ft_tag = ft_tags[0]
        self.assertEqual(ft_tag['rel'], str(ft1.id))
        self.assertEqual(ft_tag['data-filter'], str(ff1.id))
        
        
        for f in [f3, f4]:
            self.assertTrue(html.find(f.content)<0)
        
    def test_fragments_with_extra_id(self):
        ft_name = u"contacts"
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name x %}')
        html = tpl.render(Context({"ft_name": ft_name, 'x': 2}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)

        self.assertEqual(FragmentFilter.objects.count(), 1)
        self.assertEqual(FragmentFilter.objects.filter(extra_id='2').count(), 1)
        
    def test_view_fragments_name_as_string(self):
        ft1 = mommy.make(FragmentType, name="contacts")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty")
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh")
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments "contacts" %}')
        html = tpl.render(Context({"ft_name": "contacts"}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f1, f2, f3]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_fragments_name_and_extra_id_as_string(self):
        ft1 = mommy.make(FragmentType, name="contacts")
        ff1 = mommy.make(FragmentFilter, extra_id="hello")
        ff2 = mommy.make(FragmentFilter, extra_id="2")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty", filter=ff1)
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh", filter=ff1)
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn", filter=ff2)
        f4 = mommy.make(Fragment, type=ft1, content="Zsxdrg", filter=None)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments "contacts" "hello" %}')
        html = tpl.render(Context({"ft_name": "contacts"}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f1, f2]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
        for f in [f3, f4]:
            self.assertTrue(html.find(f.content)<0)
        
    def test_view_fragments_order(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty", position=3)
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh", position=1)
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn", position=2)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f2, f3, f1]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
    def test_view_only_specified_fragments(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        ft2 = mommy.make(FragmentType, name="AAAA")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty")
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh")
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn")
        
        g1 = mommy.make(Fragment, type=ft2, content="POIUYT")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f2, f3, f1]]
        for pos in positions:
            self.assertTrue(pos>=0)
        
        positions = [html.find('{0}'.format(f.content)) for f in [g1]]
        for pos in positions:
            self.assertTrue(pos==-1)
            
    def test_view_only_specified_fragments_extra_id(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        ft2 = mommy.make(FragmentType, name="AAAA")
        
        ff1 = mommy.make(FragmentFilter, extra_id="hello")
        ff2 = mommy.make(FragmentFilter, extra_id="2")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty", filter=ff1)
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh", filter=ff1)
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn", filter=ff2)
        f4 = mommy.make(Fragment, type=ft1, content="Zsxdrg", filter=None)
        
        g1 = mommy.make(Fragment, type=ft2, content="POIUYT", filter=ff1)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name "hello" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        positions = [html.find('{0}'.format(f.content)) for f in [f2, f1]]
        for pos in positions:
            self.assertTrue(pos>=0)
        
        positions = [html.find('{0}'.format(f.content)) for f in [g1, f3, f4]]
        for pos in positions:
            self.assertTrue(pos==-1)
            
    def test_view_fragments_edit_mode(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        ft2 = mommy.make(FragmentType, name="AAAA")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty")
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh")
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn")
        
        g1 = mommy.make(Fragment, type=ft2, content="POIUYT")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name %}')
        html = tpl.render(Context({"ft_name": ft_name, "form": True}))
        
        positions = [html.find(self.editable_field_tpl.format(f.id, f.content)) for f in [f1, f2, f3]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
        positions = [html.find(self.editable_field_tpl.format(f.id, f.content)) for f in [g1]]
        for pos in positions:
            self.assertTrue(pos==-1)
            
    def test_view_fragments_extra_id_edit_mode(self):
        ft_name = u"contacts"
        ft1 = mommy.make(FragmentType, name=ft_name)
        ft2 = mommy.make(FragmentType, name="AAAA")
        
        ff1 = mommy.make(FragmentFilter, extra_id="hello")
        ff2 = mommy.make(FragmentFilter, extra_id="2")
        
        f1 = mommy.make(Fragment, type=ft1, content="Azerty", filter=ff1)
        f2 = mommy.make(Fragment, type=ft1, content="Qsdfgh", filter=ff1)
        f3 = mommy.make(Fragment, type=ft1, content="Wxcvbn", filter=ff2)
        f4 = mommy.make(Fragment, type=ft1, content="Zsxdrg", filter=None)
        
        g1 = mommy.make(Fragment, type=ft2, content="POIUYT")
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name "hello" %}')
        html = tpl.render(Context({"ft_name": ft_name, "form": True}))
        
        positions = [html.find(self.editable_field_tpl.format(f.id, f.content)) for f in [f1, f2]]
        for pos in positions:
            self.assertTrue(pos>=0)
        sorted_positions = positions[:]
        sorted_positions.sort()
        self.assertEqual(positions, sorted_positions)
        
        positions = [html.find(self.editable_field_tpl.format(f.id, f.content)) for f in [g1, f3, f4]]
        for pos in positions:
            self.assertTrue(pos==-1)
            
    def test_fragments_with_template(self):
        ft_name = u"contacts"
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(0, len(soup.select('.panel')))
        
    def test_view_fragments_with_template(self):
        ft_name = u"contacts"
        ft = mommy.make(FragmentType, name=ft_name)
        
        f = mommy.make(Fragment, type=ft)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(1, len(soup.select('.panel')))
        
    def test_view_fragments_with_template_edit_mode(self):
        ft_name = u"contacts"
        ft = mommy.make(FragmentType, name=ft_name)
        
        f = mommy.make(Fragment, type=ft)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name, 'form': True}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        
        soup = BeautifulSoup(html)
        self.assertEqual(1, len(soup.select('.panel')))
        self.assertEqual(1, len(soup.select('.panel input')))
        self.assertEqual(1, len(soup.select('.panel .djaloha-editable')))
    
    def test_view_fragments_with_template2(self):
        ft_name = u"contacts"
        ft = mommy.make(FragmentType, name=ft_name)
        
        f = mommy.make(Fragment, type=ft)
        f = mommy.make(Fragment, type=ft)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        soup = BeautifulSoup(html)
        self.assertEqual(2, len(soup.select('.panel')))
        
    def test_view_fragments_with_template3(self):
        ft_name = u"contacts"
        ft = mommy.make(FragmentType, name=ft_name)
        
        f = mommy.make(Fragment, type=ft)
        f = mommy.make(Fragment, type=ft)
        
        tpl = Template('{% load coop_edition %}{% coop_fragments ft_name template_name="test/_fragment.html" %}')
        html = tpl.render(Context({"ft_name": ft_name, 'form': True}))
        
        self.assertEqual(FragmentType.objects.count(), 1)
        self.assertEqual(FragmentType.objects.filter(name=ft_name).count(), 1)
        soup = BeautifulSoup(html)
        self.assertEqual(3, len(soup.select('.panel'))) # 1 extra panel if_cms_edition and fragment index > 0
    
    def _log_as_editor(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
        
        ct1 = ContentType.objects.get_for_model(get_article_class())
        ct2 = ContentType.objects.get_for_model(Fragment)
        
        for ct in (ct1, ct2):
            
            perm = 'change_{0}'.format(ct.model)
            can_edit = Permission.objects.get(content_type=ct, codename=perm)
            user.user_permissions.add(can_edit)
            
            perm = 'add_{0}'.format(ct.model)
            can_add = Permission.objects.get(content_type=ct, codename=perm)
            user.user_permissions.add(can_add)
        
        user.is_active = True
        user.save()
        return self.client.login(username='toto', password='toto')
    
    def _log_as_regular_user(self):
        user = User.objects.create_user('titi', 'titi@toto.fr', 'titi')
        
        ct = ContentType.objects.get_for_model(get_article_class())
        
        user.is_active = True
        user.save()
        return self.client.login(username='titi', password='titi')
        
    
    def _check_article(self, response, data):
        for (key, value) in data.items():
            self.assertContains(response, value)
    #        
    #def _check_article_not_changed(self, article, data, initial_data):
    #    article = get_article_class().objects.get(id=article.id)
    #
    #    for (key, value) in data.items():
    #        self.assertNotEquals(getattr(article, key), value)
    #        
    #    for (key, value) in initial_data.items():
    #        self.assertEquals(getattr(article, key), value)

    def test_view_article_no_fragments(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, FragmentType.objects.count())
        self.assertEqual("parts", FragmentType.objects.all()[0].name)
        
    def test_view_article_with_fragments(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        f1 = mommy.make(Fragment, type=ft, content="Azertyuiop")
        
        response = self.client.get(article.get_absolute_url())
        
        self.assertEqual(200, response.status_code)
        self.assertContains(response, f1.content)
        
    def test_view_article_with_fragments_extra_id(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        ff1 = mommy.make(FragmentFilter, extra_id=str(article.id))
        ff2 = mommy.make(FragmentFilter, extra_id="hello")
        f1 = mommy.make(Fragment, type=ft, content="Azertyuiop", filter=ff1)
        f2 = mommy.make(Fragment, type=ft, content="QSDFGHJKLM", filter=ff2)
        f3 = mommy.make(Fragment, type=ft, content="Wxcvbn,;:=", filter=None)
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, f1.content)
        self.assertNotContains(response, f2.content)
        self.assertNotContains(response, f3.content)
        
    def test_view_article_with_fragment_with_css(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        f1 = mommy.make(Fragment, type=ft, content="Azertyuiop", css_class="this-is-my-fragment")
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, f1.content)
        
        soup = BeautifulSoup(response.content)
        fragment = soup.select("div."+f1.css_class)[0]
        self.assertEqual(f1.content, fragment.text)
        
    def test_edit_article_no_fragments(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        data = {"title": 'salut', 'content': 'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
        data = {"title": 'bye', 'content': 'au revoir'}
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self._check_article(response, data)
        
    def test_edit_article_with_fragments(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        f1 = mommy.make(Fragment, type=ft, content="Azertyuiop")
        
        new_f1_content = u"Qsdfghjklm"
        data = {
            "title": 'salut',
            'content': 'bonjour!',
            'djaloha__coop_cms__Fragment__id__{0}__content'.format(f1.id): new_f1_content,
        }
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, data['title'])
        self.assertContains(response, data['content'])
        self.assertContains(response, new_f1_content)
        
    def test_edit_article_with_fragments_extra_id(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        ff = mommy.make(FragmentFilter, extra_id=str(article.id))
        f1 = mommy.make(Fragment, type=ft, content="Azertyuiop", filter=ff)
        
        new_f1_content = u"Qsdfghjklm"
        data = {
            "title": 'salut',
            'content': 'bonjour!',
            'djaloha__coop_cms__Fragment__id__{0}__content'.format(f1.id): new_f1_content,
        }
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, data['title'])
        self.assertContains(response, data['content'])
        self.assertContains(response, new_f1_content)
        
    def test_view_add_fragment(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
    def test_view_add_fragment_check_filters(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[1][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        #url = reverse("coop_cms_add_fragment")
        url = article.get_edit_url()
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        ft_tags = soup.select(".coop-fragment-type")
        ft_objs = FragmentType.objects.all()
        ff_objs = FragmentFilter.objects.all()
        
        self.assertEqual(len(ft_tags), 2)
        self.assertEqual(ft_objs.count(), 2)
        self.assertEqual(ff_objs.count(), 1)
        
        for i in range(2):
            self.assertEqual(int(ft_tags[i]["rel"]), ft_objs[i].id)
        
        self.assertEqual(ft_tags[0]["data-filter"], '')
        self.assertEqual(ft_tags[1]["data-filter"], str(ff_objs[0].id))
        
        
    def test_view_add_fragment_no_filter_check_filters(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        #url = reverse("coop_cms_add_fragment")
        url = article.get_edit_url()
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        
        ft_tags = soup.select(".coop-fragment-type")
        ft_objs = FragmentType.objects.all()
        ff_objs = FragmentFilter.objects.all()
        
        self.assertEqual(len(ft_tags), 1)
        self.assertEqual(ft_objs.count(), 1)
        self.assertEqual(ff_objs.count(), 0)
        
        self.assertEqual(int(ft_tags[0]["rel"]), ft_objs[0].id)
        
        self.assertEqual(ft_tags[0]["data-filter"], '')
        
    def test_view_add_fragment_permission_denied(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        
        self._log_as_regular_user()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
        
    def _add_fragment(self, data, errors_count=0):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        if errors_count:
            self.assertEqual(errors_count, len(errs))
        else:
            self.assertEqual([], errs)
            expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
            self.assertEqual(response.content, expected)
        
        return response        
        
    def test_add_fragment(self):
        ft = mommy.make(FragmentType, name="parts")
        data = {
            'type': ft.id,
            'name': 'abcd',
            'position': 0,
            'filter': '',
        }
        
        response = self._add_fragment(data)
        f = Fragment.objects.all()[0]
        
        self.assertEqual(f.type, ft)
        self.assertEqual(f.name, data['name'])
        self.assertEqual(f.css_class, '')   
        self.assertEqual(f.position, 1)
        self.assertEqual(f.filter, None)

        
    def test_add_fragment_filter(self):
        ft = mommy.make(FragmentType, name="parts")
        ff = mommy.make(FragmentFilter, extra_id="2")
        data = {
            'type': ft.id,
            'name': 'abcd',
            'position': 0,
            'filter': ff.id
        }
        
        response = self._add_fragment(data)
        f = Fragment.objects.all()[0]
        
        self.assertEqual(f.type, ft)
        self.assertEqual(f.name, data['name'])
        self.assertEqual(f.css_class, '')   
        self.assertEqual(f.position, 1)
        self.assertEqual(f.filter, ff)
        
    def test_add_fragment_position(self):
        ft = mommy.make(FragmentType, name="parts")
        data = {
            'type': ft.id,
            'name': 'abcd',
            'position': 2,
            'filter': '',
        }
        
        response = self._add_fragment(data)
        f = Fragment.objects.all()[0]
        
        self.assertEqual(f.type, ft)
        self.assertEqual(f.name, data['name'])
        self.assertEqual(f.css_class, '')
        self.assertEqual(f.position, 2)
        
    def test_add_fragment_invalid_filter(self):
        ft = mommy.make(FragmentType, name="parts")
        data = {
            'type': ft.id,
            'name': 'abcd',
            'position': 2,
            'filter': '0',
        }
        
        response = self._add_fragment(data, 1)
        self.assertEqual(0, Fragment.objects.count())
        
    def test_add_fragment_css(self):
        ft = mommy.make(FragmentType, name="parts")
        data = {
            'type': ft.id,
            'name': 'abcd',
            'css_class': 'okidki',
            'position': 0,
        }
        
        response = self._add_fragment(data)
        f = Fragment.objects.all()[0]
        
        self.assertEqual(f.type, ft)
        self.assertEqual(f.name, data['name'])
        self.assertEqual(f.css_class, '')   
        self.assertEqual(f.position, 1)
            
    def test_view_add_fragment_permission_denied(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft = mommy.make(FragmentType, name="parts")
        data = {
            'type': ft,
            'name': 'abcd',
            'css_class': 'okidoki',
            'position': 0,
        }
        
        url = reverse("coop_cms_add_fragment")
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        next_url = "http://testserver/accounts/login/?next={0}".format(url)
        self.assertEqual(next_url, response['Location'])
        
        self._log_as_regular_user()
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(403, response.status_code)
        
        self.assertEqual(0, Fragment.objects.count())
        
    def test_view_edit_fragments_empty(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        
    def test_view_edit_fragments(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        f1 = mommy.make(Fragment, name="azerty")
        f2 = mommy.make(Fragment, name="qwerty")
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(200, response.status_code)
        self.assertContains(response, f1.name)
        self.assertContains(response, f2.name)
        
    def test_view_edit_fragments_permission_denied(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.get(url)
        
        self.assertEqual(302, response.status_code)
        
        self._log_as_regular_user()
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)    
    
    def test_edit_fragment(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(2, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        f2 = Fragment.objects.get(id=f2.id)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty!")
        self.assertEqual(f1.css_class, "")   
        self.assertEqual(f1.position, 5)
    
    
        self.assertEqual(f2.type, ft2)
        self.assertEqual(f2.name, "qwerty+")
        self.assertEqual(f2.css_class, "")   
        self.assertEqual(f2.position, 2)

    def test_edit_fragment_css_allowed(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType, allowed_css_classes="oups")
        ft2 = mommy.make(FragmentType, allowed_css_classes="aaa,bbb")
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(2, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        f2 = Fragment.objects.get(id=f2.id)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty!")
        self.assertEqual(f1.css_class, "oups")   
        self.assertEqual(f1.position, 5)
    
    
        self.assertEqual(f2.type, ft2)
        self.assertEqual(f2.name, "qwerty+")
        self.assertEqual(f2.css_class, "aaa")   
        self.assertEqual(f2.position, 2)
  
    def test_edit_fragment_css_not_allowed(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType, allowed_css_classes="")
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(2, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        f2 = Fragment.objects.get(id=f2.id)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty!")
        self.assertEqual(f1.css_class, "")   
        self.assertEqual(f1.position, 5)
    
    
        self.assertEqual(f2.type, ft2)
        self.assertEqual(f2.name, "qwerty+")
        self.assertEqual(f2.css_class, "")   
        self.assertEqual(f2.position, 2)
  
      
    def test_edit_fragment_delete(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "",
            'form-1-position': 2,
            'form-1-delete_me': True,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual([], errs)
        
        expected = u'<script>$.colorbox.close(); window.location=window.location;</script>'.format()
        self.assertEqual(response.content, expected)
        
        self.assertEqual(1, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        self.assertEqual(Fragment.objects.filter(id=f2.id).count(), 0)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty!")
        self.assertEqual(f1.css_class, "")   
        self.assertEqual(f1.position, 5)
        
    def test_edit_fragment_invalid_position(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': "AAA",
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(1, len(errs))
    
    def test_edit_fragment_empty_name(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': "",
            'form-0-css_class': "oups",
            'form-0-position': 1,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        self._log_as_editor()
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        
        soup = BeautifulSoup(response.content)
        errs = soup.select("ul.errorlist li")
        self.assertEqual(1, len(errs))
        
    
    def test_edit_fragment_permission_denied(self):
        template = settings.COOP_CMS_ARTICLE_TEMPLATES[0][0]
        article = get_article_class().objects.create(title="test", template=template, publication=BaseArticle.PUBLISHED)
        
        ft1 = mommy.make(FragmentType)
        ft2 = mommy.make(FragmentType)
        f1 = mommy.make(Fragment, name="azerty", type=ft1)
        f2 = mommy.make(Fragment, name="qwerty", type=ft2)
        
        data = {
            'form-0-id': f1.id,
            'form-0-type': f1.type.id,
            'form-0-name': f1.name+"!",
            'form-0-css_class': "oups",
            'form-0-position': 5,
            'form-0-delete_me': False,
            
            'form-1-id': f2.id,
            'form-1-type': f2.type.id,
            'form-1-name': f2.name+"+",
            'form-1-css_class': "aaa",
            'form-1-position': 2,
            'form-1-delete_me': False,
            
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 2,
            'form-MAX_NUM_FORMS': 2
        }
        
        url = reverse("coop_cms_edit_fragments")
        response = self.client.post(url, data=data, follow=False)
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(2, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        f2 = Fragment.objects.get(id=f2.id)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty")
        self.assertEqual(f1.css_class, "")   
        self.assertEqual(f1.position, 1)
    
        self.assertEqual(f2.type, ft2)
        self.assertEqual(f2.name, "qwerty")
        self.assertEqual(f2.css_class, "")   
        self.assertEqual(f2.position, 1)

        self._log_as_regular_user()
        response = self.client.post(url, data=data)
        self.assertEqual(403, response.status_code)  
        
        self.assertEqual(2, Fragment.objects.count())
        f1 = Fragment.objects.get(id=f1.id)
        f2 = Fragment.objects.get(id=f2.id)

        self.assertEqual(f1.type, ft1)
        self.assertEqual(f1.name, "azerty")
        self.assertEqual(f1.css_class, "")   
        self.assertEqual(f1.position, 1)
    
        self.assertEqual(f2.type, ft2)
        self.assertEqual(f2.name, "qwerty")
        self.assertEqual(f2.css_class, "")   
        self.assertEqual(f2.position, 1)



# -*- coding:utf-8 -*-
"""sitemaps"""

from django.db.models import Q
from django.conf import settings
from django.conf.urls import url
from django.contrib.sitemaps import Sitemap
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from coop_cms.models import BaseArticle, SiteSettings
from coop_cms.settings import get_article_class, is_localized

if is_localized():
    from localeurl.sitemaps import LocaleurlSitemap # pylint: disable=F0401
    base_sitemap_class = LocaleurlSitemap
else:
    base_sitemap_class = Sitemap


class ViewSitemap(Sitemap):
    """Sitemap base class for django view"""
    view_names = []
    
    def items(self):
        """get items"""
        class Klass(object):
            def __init__(self, name):
                self.name = name
            
            def get_absolute_url(self):
                return reverse(self.name)
            
        return [Klass(x) for x in self.view_names]


class BaseSitemap(base_sitemap_class):
    _current_site = None

    def __init__(self, language, site):
        if is_localized():
            super(BaseSitemap, self).__init__(language)
        else:
            super(BaseSitemap, self).__init__()
        self._site = site

    def get_urls(self, page=1, site=None, protocol=None):
        return super(BaseSitemap, self).get_urls(page, self._site, protocol=protocol)

    def get_current_site(self):
        if not self._current_site:
            self._current_site = Site.objects.get_current()
        return self._current_site


class ArticleSitemap(BaseSitemap):
    """article sitemap"""
    changefreq = "weekly"
    priority = 0.5
    _sitemap_mode = None

    def get_sitemap_mode(self):
        site = self.get_current_site()
        if not self._sitemap_mode:
            try:
                self._sitemap_mode = site.sitesettings.sitemap_mode
            except SiteSettings.DoesNotExist:
                self._sitemap_mode = SiteSettings.SITEMAP_ONLY_SITE
        return self._sitemap_mode

    def items(self):
        """items"""
        article_class = get_article_class()

        sitemap_mode = self.get_sitemap_mode()

        queryset = article_class.objects.filter(publication=BaseArticle.PUBLISHED)

        if self._site == self.get_current_site():
            return queryset.filter(sites=self._site)
        else:
            if sitemap_mode == SiteSettings.SITEMAP_ONLY_SITE:
                #Only the articles of current sites
                return article_class.objects.none()
            elif sitemap_mode == SiteSettings.SITEMAP_ALL:
                #Articles of which are only on the site and not in current site
                return queryset.filter(sites=self._site).exclude(sites=self.get_current_site())

    def lastmod(self, obj):
        """item last modification"""
        return obj.modified


def get_sitemaps(langs=None):
    """return sitemaps"""
    sitemaps = {}
    sites = Site.objects.all()
    if is_localized():
        lang_codes = langs or [code for (code, _x) in settings.LANGUAGES]
        for code in lang_codes:
            for site in sites:
                site_suffix = "_{0}_{1}".format(site.id, code)
                sitemaps['coop_cms{0}'.format(site_suffix)] = ArticleSitemap(code, site)
    else:
        for site in sites:
            site_suffix = "_{0}".format(site.id)
            sitemaps['coop_cms{0}'.format(site_suffix)] = ArticleSitemap(None, site)
    return sitemaps


urlpatterns = (
    url(
        r'^sitemap\.xml$',
        'django.contrib.sitemaps.views.sitemap',
        {'sitemaps': get_sitemaps},
        name="coop_cms_sitemap"
    ),
)



# -*- coding: utf-8 -*-
"""views"""

from datetime import datetime
import itertools
import json
import logging
import mimetypes
import os.path
import sys
import unicodedata
from urlparse import urlparse

from django.db.models import Q
from django.db.models.aggregates import Max
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.contrib import messages
from django.contrib.messages.api import success as success_message, error as error_message
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError, PermissionDenied
from django.core.servers.basehttp import FileWrapper
from django.core.urlresolvers import reverse
from django.forms.models import modelformset_factory
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from django.middleware.csrf import REASON_NO_REFERER, REASON_NO_CSRF_COOKIE
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext, Context, TemplateDoesNotExist
from django.template.loader import get_template, select_template
from django.utils.translation import ugettext as _, check_for_language, activate
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView

from djaloha import utils as djaloha_utils
from colorbox.decorators import popup_redirect, popup_close

from coop_cms import forms
from coop_cms import models
from coop_cms.generic_views import EditableObjectView
from coop_cms.settings import (
    cms_no_homepage, get_article_class, get_article_form, get_article_settings_form,
    get_navtree_class, get_new_article_form, get_newsletter_form
)
from coop_cms.shortcuts import get_article_or_404, get_headlines, redirect_if_alias
from coop_cms.utils import send_newsletter

logger = logging.getLogger("coop_cms")


def get_article_template(article):
    """get article template"""
    template = article.template
    if not template:
        template = 'coop_cms/article.html'
    return template


def tree_map(request):
    """tree map"""
    return render_to_response(
        'coop_cms/tree_map.html',
        #{'tree': models.get_navTree_class().objects.get(id=tree_id)},  # what is the default tree for the site
        RequestContext(request)
    )


def homepage(request):
    """view homepage"""
    try:
        if cms_no_homepage():
            raise Http404
        
        site = Site.objects.get_current()
        
        #Try site settings
        try:
            site_settings = models.SiteSettings.objects.get(site=site)
            if site_settings.homepage_url:
                return HttpResponseRedirect(site_settings.homepage_url)
        except models.SiteSettings.DoesNotExist:
            pass
        
        #Try: homepage article #Deprecated
        article = get_article_class().objects.get(homepage_for_site=site, sites=site.id)
        return HttpResponseRedirect(article.get_absolute_url())
    except get_article_class().DoesNotExist:
        return HttpResponseRedirect(reverse('coop_cms_view_all_articles'))


@login_required
def view_all_articles(request):
    """all article"""

    articles_admin_url = newsletters_admin_url = add_article_url = add_newsletter_url = None

    if request.user.is_staff:
        article_class = get_article_class()
        view_name = 'admin:{0}_{1}_changelist'.format(article_class._meta.app_label, article_class._meta.module_name)
        articles_admin_url = reverse(view_name)

        newsletters_admin_url = reverse('admin:coop_cms_newsletter_changelist')

        add_newsletter_url = reverse('admin:coop_cms_newsletter_add')

    article_class = get_article_class()
    content_type = ContentType.objects.get_for_model(article_class)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
    if request.user.has_perm(perm):
        add_article_url = reverse('coop_cms_new_article')

    return render_to_response(
        'coop_cms/view_all_articles.html',
        {
            'articles': article_class.objects.filter(sites__id=settings.SITE_ID).order_by('-id')[:10],
            'newsletters': models.Newsletter.objects.all().order_by('-id')[:10],
            'editable': True,
            'articles_list_url': articles_admin_url,
            'newsletters_list_url': newsletters_admin_url,
            'add_article_url': add_article_url,
            'add_newsletter_url': add_newsletter_url,
        },
        RequestContext(request)
    )


@login_required
@popup_redirect
def set_homepage(request, article_id):
    """use the article as homepage"""
    article = get_object_or_404(get_article_class(), id=article_id)

    if not request.user.has_perm('can_edit_article', article):
        raise PermissionDenied

    if request.method == "POST":
        article.homepage_for_site = Site.objects.get(id=settings.SITE_ID)
        article.save()
        return HttpResponseRedirect(reverse('coop_cms_homepage'))

    context_dict = {
        'article': article,
        'title': _(u"Do you want to use this article as homepage?"),
    }

    return render_to_response(
        'coop_cms/popup_set_homepage.html',
        context_dict,
        context_instance=RequestContext(request)
    )


def view_article(request, url, extra_context=None, force_template=None):
    """view the article"""
    try:
        not_archived = Q(publication=models.BaseArticle.ARCHIVED)
        article = get_article_or_404(Q(slug=url) & Q(sites=settings.SITE_ID) & ~not_archived) #Draft & Published
    except Http404:
        return redirect_if_alias(path=url)
    
    if not request.user.has_perm('can_view_article', article):
        raise PermissionDenied()

    editable = request.user.has_perm('can_edit_article', article)
    
    context_dict = {
        'editable': editable, 'edit_mode': False, 'article': article,
        'draft': article.publication == models.BaseArticle.DRAFT,
        'headlines': get_headlines(article, editable=editable),
    }
    
    if extra_context:
        context_dict.update(extra_context)

    return render_to_response(
        force_template or get_article_template(article),
        context_dict,
        context_instance=RequestContext(request)
    )


@login_required
def edit_article(request, url, extra_context=None, force_template=None):
    """edit the article"""
    
    article_form_class = get_article_form()

    article = get_article_or_404(slug=url, sites=settings.SITE_ID)
    
    if not request.user.has_perm('can_edit_article', article):
        logger.error("PermissionDenied")
        error_message(request, _(u'Permission denied'))
        raise PermissionDenied

    if request.method == "POST":
        form = article_form_class(request.POST, request.FILES, instance=article)

        forms_args = djaloha_utils.extract_forms_args(request.POST)
        djaloha_forms = djaloha_utils.make_forms(forms_args, request.POST)

        if form.is_valid() and all([f.is_valid() for f in djaloha_forms]):
            article = form.save()
            
            if article.temp_logo:
                article.logo = article.temp_logo
                article.temp_logo = ''
                article.save()

            if djaloha_forms:
                for dj_form in djaloha_forms:
                    dj_form.save()

            success_message(request, _(u'The article has been saved properly'))

            return HttpResponseRedirect(article.get_absolute_url())
        else:
            error_text = u'<br />'.join([unicode(f.errors) for f in [form]+djaloha_forms if f.errors])
            error_message(request, _(u'An error occured: {0}'.format(error_text)))
            logger.debug("error: error_text")
    else:
        form = article_form_class(instance=article)

    context_dict = {
        'form': form,
        'editable': True, 'edit_mode': True, 'title': article.title,
        'draft': article.publication == models.BaseArticle.DRAFT, 'headlines': get_headlines(article),
        'article': article, 'ARTICLE_PUBLISHED': models.BaseArticle.PUBLISHED
    }
    
    if extra_context:
        context_dict.update(extra_context)

    return render_to_response(
        force_template or get_article_template(article),
        context_dict,
        context_instance=RequestContext(request)
    )


@login_required
def cancel_edit_article(request, url):
    """if cancel_edit, delete the preview image"""
    article = get_article_or_404(slug=url, sites=settings.SITE_ID)
    if article.temp_logo:
        article.temp_logo = ''
        article.save()
    return HttpResponseRedirect(article.get_absolute_url())


@login_required
@popup_redirect
def publish_article(request, url):
    """change the publication status of an article"""
    article = get_article_or_404(slug=url, sites=settings.SITE_ID)

    if not request.user.has_perm('can_publish_article', article):
        raise PermissionDenied

    draft = (article.publication == models.BaseArticle.DRAFT)
    if draft:
        article.publication = models.BaseArticle.PUBLISHED
    else:
        article.publication = models.BaseArticle.DRAFT

    if request.method == "POST":
        form = forms.PublishArticleForm(request.POST, instance=article)
        if form.is_valid():
            article = form.save()
            return HttpResponseRedirect(article.get_absolute_url())
    else:
        form = forms.PublishArticleForm(instance=article)

    context_dict = {
        'form': form,
        'article': article,
        'draft': draft,
        'title': _(u"Do you want to publish this article?") if draft else _(u"Make it draft?"),
    }

    return render_to_response(
        'coop_cms/popup_publish_article.html',
        context_dict,
        context_instance=RequestContext(request)
    )


@login_required
def show_media(request, media_type):
    """show media library"""
    try:
        if not request.user.is_staff:
            raise PermissionDenied
        
        is_ajax = request.GET.get('page', 0)
        media_filter = request.GET.get('media_filter', 0)
    
        if request.session.get("coop_cms_media_doc", False):
            media_type = 'document' #force the doc
            del request.session["coop_cms_media_doc"]
    
        if media_type == 'image':
            queryset = models.Image.objects.all().order_by("ordering", "-created")
            context = {
                'media_url': reverse('coop_cms_media_images'),
                'media_slide_template': 'coop_cms/slide_images_content.html',
            }
        else:
            media_type = "document"
            queryset = models.Document.objects.all().order_by("ordering", "-created")
            context = {
                'media_url': reverse('coop_cms_media_documents'),
                'media_slide_template': 'coop_cms/slide_docs_content.html',
            }
            
        context['is_ajax'] = is_ajax
        context['media_type'] = media_type
        
        media_filters = [media.filters.all() for media in queryset.all()] # list of lists of media_filters
        media_filters = itertools.chain(*media_filters) #flat list of media_filters
        context['media_filters'] = sorted(
            list(set(media_filters)), key=lambda mf: mf.name.upper()
        )#flat list of unique media filters sorted by alphabetical order (ignore case)
        
        if int(media_filter):
            queryset = queryset.filter(filters__id=media_filter)
            context['media_filter'] = int(media_filter)
        context[media_type+'s'] = queryset
        
        template = get_template('coop_cms/slide_base.html')
        html = template.render(RequestContext(request, context))
    
        if is_ajax:
            data = {
                'html': html,
                'media_type': media_type,
            }
            return HttpResponse(json.dumps(data), content_type="application/json")
        else:
            return HttpResponse(html)
    except Exception:
        logger.exception("show_media")
        raise


@login_required
def upload_image(request):
    """upload image"""

    try:
        if not request.user.has_perm("coop_cms.add_image"):
            raise PermissionDenied()
        
        if request.method == "POST":
            form = forms.AddImageForm(request.POST, request.FILES)
            if form.is_valid():
                src = form.cleaned_data['image']
                descr = form.cleaned_data['descr']
                if not descr:
                    descr = os.path.splitext(src.name)[0]
                image = models.Image(name=descr)
                image.size = form.cleaned_data["size"]
                image.file.save(src.name, src)
                image.save()
                
                filters = form.cleaned_data['filters']
                if filters:
                    image.filters.add(*filters)
                    image.save()
                
                return HttpResponse("close_popup_and_media_slide")
        else:
            form = forms.AddImageForm()
    
        return render_to_response(
            'coop_cms/popup_upload_image.html',
            locals(),
            context_instance=RequestContext(request)
        )
    except Exception:
        logger.exception("upload_image")
        raise
        

@login_required
def upload_doc(request):
    """upload document"""
    try:
        if not request.user.has_perm("coop_cms.add_document"):
            raise PermissionDenied()
        
        if request.method == "POST":
            form = forms.AddDocForm(request.POST, request.FILES)
            if form.is_valid():
                doc = form.save()
                if not doc.name:
                    doc.name = os.path.splitext(os.path.basename(doc.file.name))[0]
                    doc.save()
    
                request.session["coop_cms_media_doc"] = True
    
                return HttpResponse("close_popup_and_media_slide")
        else:
            form = forms.AddDocForm()
    
        return render_to_response(
            'coop_cms/popup_upload_doc.html',
            locals(),
            context_instance=RequestContext(request)
        )
    except Exception:
        logger.exception("upload_doc")
        raise


@login_required
@popup_redirect
def change_template(request, article_id):
    """change template"""

    article = get_object_or_404(get_article_class(), id=article_id)
    if request.method == "POST":
        form = forms.ArticleTemplateForm(article, request.user, request.POST, request.FILES)
        if form.is_valid():
            article.template = form.cleaned_data['template']
            article.save()
            return HttpResponseRedirect(article.get_edit_url())
    else:
        form = forms.ArticleTemplateForm(article, request.user)

    return render_to_response(
        'coop_cms/popup_change_template.html',
        locals(),
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def article_settings(request, article_id):
    """article settings"""
    article = get_object_or_404(get_article_class(), id=article_id)
    article_settings_form_class = get_article_settings_form()
    
    if not request.user.has_perm('can_edit_article', article):
        raise PermissionDenied
    
    if request.method == "POST":
        form = article_settings_form_class(request.user, request.POST, request.FILES, instance=article)
        if form.is_valid():
            article = form.save()
            form.save_m2m()
            return HttpResponseRedirect(article.get_absolute_url())
    else:
        form = article_settings_form_class(request.user, instance=article)

    context = {
        'article': article,
        'form': form,
    }
    return render_to_response(
        'coop_cms/popup_article_settings.html',
        context,
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def new_article(request):
    """new article"""
    article_class = get_article_class()
    new_article_form = get_new_article_form()
    
    content_type = ContentType.objects.get_for_model(article_class)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not request.user.has_perm(perm):
        raise PermissionDenied

    if request.method == "POST":
        form = new_article_form(request.user, request.POST, request.FILES)
        if form.is_valid():
            article = form.save()
            form.save_m2m()
            success_message(request, _(u'The article has been created properly'))
            return HttpResponseRedirect(article.get_edit_url())
    else:
        form = new_article_form(request.user)

    return render_to_response(
        'coop_cms/popup_new_article.html',
        locals(),
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def new_link(request):
    """new link"""
    content_type = ContentType.objects.get_for_model(models.Link)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not request.user.has_perm(perm):
        raise PermissionDenied

    if request.method == "POST":
        form = forms.NewLinkForm(request.POST)
        if form.is_valid():
            form.save()
            homepage_url = reverse('coop_cms_homepage')
            next_url = request.META.get('HTTP_REFERER', homepage_url)
            success_message(request, _(u'The link has been created properly'))
            return HttpResponseRedirect(next_url)
    else:
        form = forms.NewLinkForm()
        
    context = {
        'form': form,
    }

    return render_to_response(
        'coop_cms/popup_new_link.html',
        context,
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def new_newsletter(request, newsletter_id=None):
    """new newsletter"""

    if newsletter_id:
        newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)
    else:
        newsletter = None
        
    if request.method == "POST":
        form = forms.NewNewsletterForm(request.user, request.POST, instance=newsletter)
        if form.is_valid():
            newsletter = form.save()
            return HttpResponseRedirect(newsletter.get_edit_url())
    else:
        form = forms.NewNewsletterForm(request.user, instance=newsletter)

    return render_to_response(
        'coop_cms/popup_new_newsletter.html',
        locals(),
        context_instance=RequestContext(request)
    )


@login_required
def update_logo(request, article_id):
    """update logo"""
    try:
        article = get_object_or_404(get_article_class(), id=article_id)
        if request.method == "POST":
            form = forms.ArticleLogoForm(request.POST, request.FILES)
            if form.is_valid():
                article.temp_logo = form.cleaned_data['image']
                article.save()
                url = article.logo_thumbnail(True).url
                data = {'ok': True, 'src': url}
                return HttpResponse(json.dumps(data), content_type='application/json')
            else:
                template = get_template('coop_cms/popup_update_logo.html')
                html = template.render(Context(locals()))
                data = {'ok': False, 'html': html}
                return HttpResponse(json.dumps(data), content_type='application/json')
        else:
            form = forms.ArticleLogoForm()
    
        return render_to_response(
            'coop_cms/popup_update_logo.html',
            locals(),
            context_instance=RequestContext(request)
        )
    except Exception:
        logger.exception("update_logo")
        raise


@login_required
def download_doc(request, doc_id):
    """download a doc"""
    doc = get_object_or_404(models.Document, id=doc_id)
    if not request.user.has_perm('can_download_file', doc):
        raise PermissionDenied
    
    if 'filetransfers' in settings.INSTALLED_APPS:
        from filetransfers.api import serve_file # pylint: disable=F0401
        return serve_file(request, doc.file)
    else:
        #legacy version just kept for compatibility if filetransfers is not installed
        logger.warning("install django-filetransfers for better download support")
        the_file = doc.file
        the_file.open('rb')
        wrapper = FileWrapper(the_file)
        mime_type = mimetypes.guess_type(the_file.name)[0]
        if not mime_type:
            mime_type = u'application/octet-stream'
        response = HttpResponse(wrapper, content_type=mime_type)
        response['Content-Length'] = the_file.size
        filename = unicodedata.normalize('NFKD', os.path.split(the_file.name)[1]).encode("utf8", 'ignore')
        filename = filename.replace(' ', '-')
        response['Content-Disposition'] = 'attachment; filename={0}'.format(filename)
        return response

#navigation tree --------------------------------------------------------------


def view_navnode(request, tree):
    """show info about the node when selected"""
    try:
        response = {}
    
        node_id = request.POST['node_id']
        node = models.NavNode.objects.get(tree=tree, id=node_id)
        model_name = object_label = ""
    
        #get the admin url
        if node.content_type:
            app, mod = node.content_type.app_label, node.content_type.model
            admin_url = reverse("admin:{0}_{1}_change".format(app, mod), args=(node.object_id,))
    
            #load and render template for the object
            #try to load the corresponding template and if not found use the default one
            model_name = unicode(node.content_type)
            object_label = unicode(node.content_object)
            template = select_template([
                "coop_cms/navtree_content/{0}.html".format(node.content_type.name),
                "coop_cms/navtree_content/default.html"
            ])
        else:
            admin_url = u""
            template = select_template(["coop_cms/navtree_content/default.html"])
            
        html = template.render(
            RequestContext(request, {
                "node": node, "admin_url": admin_url,
                "model_name": model_name, "object_label": object_label
            })
        )
    
        #return data has dictionnary
        response['html'] = html
        response['message'] = u"Node content loaded."
    
        return response
    except Exception:
        logger.exception("view_navnode")
        raise


def rename_navnode(request, tree):
    """change the name of a node when renamed in the tree"""
    response = {}
    node_id = request.POST['node_id']
    node = models.NavNode.objects.get(tree=tree, id=node_id)  # get the node
    old_name = node.label  # get the old name for success message
    node.label = request.POST['name']  # change the name
    node.save()
    if old_name != node.label:
        response['message'] = _(u"The node '{0}' has been renamed into '{1}'.").format(old_name, node.label)
    else:
        response['message'] = ''
    return response


def remove_navnode(request, tree):
    """delete a node"""
    #Keep multi node processing even if multi select is not allowed
    response = {}
    node_ids = request.POST['node_ids'].split(";")
    for node_id in node_ids:
        models.NavNode.objects.get(tree=tree, id=node_id).delete()
    if len(node_ids) == 1:
        response['message'] = _(u"The node has been removed.")
    else:
        response['message'] = _(u"{0} nodes has been removed.").format(len(node_ids))
    return response


def move_navnode(request, tree):
    """move a node in the tree"""
    response = {}

    node_id = request.POST['node_id']
    ref_pos = request.POST['ref_pos']
    parent_id = request.POST.get('parent_id', 0)
    ref_id = request.POST.get('ref_id', 0)

    node = models.NavNode.objects.get(tree=tree, id=node_id)

    if parent_id:
        sibling_nodes = models.NavNode.objects.filter(tree=tree, parent__id=parent_id)
        parent_node = models.NavNode.objects.get(tree=tree, id=parent_id)
    else:
        sibling_nodes = models.NavNode.objects.filter(tree=tree, parent__isnull=True)
        parent_node = None

    if ref_id:
        ref_node = models.NavNode.objects.get(tree=tree, id=ref_id)
    else:
        ref_node = None

    #Update parent if changed
    if parent_node != node.parent:
        if node.parent:
            ex_siblings = models.NavNode.objects.filter(tree=tree, parent=node.parent).exclude(id=node.id)
        else:
            ex_siblings = models.NavNode.objects.filter(tree=tree, parent__isnull=True).exclude(id=node.id)

        node.parent = parent_node

        #restore ex-siblings
        for sib_node in ex_siblings.filter(ordering__gt=node.ordering):
            sib_node.ordering -= 1
            sib_node.save()

        #move siblings if inserted
        if ref_node:
            if ref_pos == "before":
                to_be_moved = sibling_nodes.filter(ordering__gte=ref_node.ordering)
                node.ordering = ref_node.ordering
            elif ref_pos == "after":
                to_be_moved = sibling_nodes.filter(ordering__gt=ref_node.ordering)
                node.ordering = ref_node.ordering + 1
            for ntbm in to_be_moved:
                ntbm.ordering += 1
                ntbm.save()

        else:
            #add at the end
            max_ordering = sibling_nodes.aggregate(max_ordering=Max('ordering'))['max_ordering'] or 0
            node.ordering = max_ordering + 1

    else:

        #Update pos if changed
        if ref_node:
            if ref_node.ordering > node.ordering:
                #move forward
                to_be_moved = sibling_nodes.filter(ordering__lt=ref_node.ordering, ordering__gt=node.ordering)
                for next_sibling_node in to_be_moved:
                    next_sibling_node.ordering -= 1
                    next_sibling_node.save()

                if ref_pos == "before":
                    node.ordering = ref_node.ordering - 1
                elif ref_pos == "after":
                    node.ordering = ref_node.ordering
                    ref_node.ordering -= 1
                    ref_node.save()

            elif ref_node.ordering < node.ordering:
                #move backward
                to_be_moved = sibling_nodes.filter(ordering__gt=ref_node.ordering, ordering__lt=node.ordering)
                for next_sibling_node in to_be_moved:
                    next_sibling_node.ordering += 1
                    next_sibling_node.save()

                if ref_pos == "before":
                    node.ordering = ref_node.ordering
                    ref_node.ordering += 1
                    ref_node.save()
                elif ref_pos == "after":
                    node.ordering = ref_node.ordering + 1

        else:
            max_ordering = sibling_nodes.aggregate(max_ordering=Max('ordering'))['max_ordering'] or 0
            node.ordering = max_ordering + 1

    node.save()
    response['message'] = _(u"The node '{0}' has been moved.").format(node.label)

    return response


def add_navnode(request, tree):
    """Add a new node"""
    response = {}

    #get the type of object
    object_type = request.POST['object_type']
    if object_type:
        app_label, model_name = object_type.split('.')
        content_type = ContentType.objects.get(app_label=app_label, model=model_name)
        model_class = content_type.model_class()
        object_id = request.POST['object_id']
        model_name = model_class._meta.verbose_name
        if not object_id:
            raise ValidationError(_(u"Please choose an existing {0}").format(model_name.lower()))
        try:
            obj = model_class.objects.get(id=object_id)
        except model_class.DoesNotExist:
            raise ValidationError(_(u"{0} {1} not found").format(model_class._meta.verbose_name, object_id))
    
        #objects can not be added twice in the navigation tree
        if models.NavNode.objects.filter(tree=tree, content_type=content_type, object_id=obj.id).count() > 0:
            raise ValidationError(_(u"The {0} is already in navigation").format(model_class._meta.verbose_name))

    else:
        content_type = None
        obj = None

    #Create the node
    parent_id = request.POST.get('parent_id', 0)
    if parent_id:
        parent = models.NavNode.objects.get(tree=tree, id=parent_id)
    else:
        parent = None
    node = models.create_navigation_node(content_type, obj, tree, parent)

    response['label'] = node.label
    response['id'] = 'node_{0}'.format(node.id)
    response['message'] = _(u"'{0}' has added to the navigation tree.").format(node.label)

    return response


def get_suggest_list(request, tree):
    """call by auto-complete"""
    response = {}
    suggestions = []
    term = request.POST["term"]  # the 1st chars entered in the autocomplete

    if tree.types.count() == 0:
        nav_types = models.NavType.objects.all()
    else:
        nav_types = tree.types.all()

    for nav_type in nav_types:
        content_type = nav_type.content_type
        if nav_type.label_rule == models.NavType.LABEL_USE_SEARCH_FIELD:
            #Get the name of the default field for the current type (eg: Page->title, Url->url ...)
            lookup = {nav_type.search_field + '__icontains': term}
            objects = content_type.model_class().objects.filter(**lookup)
        elif nav_type.label_rule == models.NavType.LABEL_USE_GET_LABEL:
            objects = [
                obj for obj in content_type.model_class().objects.all() if term.lower() in obj.get_label().lower()
            ]
        else:
            objects = [
                obj for obj in content_type.model_class().objects.all() if term.lower() in unicode(obj).lower()
            ]

        already_in_navigation = [
            node.object_id for node in models.NavNode.objects.filter(tree=tree, content_type=content_type)
        ]

        #Get suggestions as a list of {label: object.get_label() or unicode if no get_label, 'value':<object.id>}
        for obj in objects:
            if obj.id not in already_in_navigation:
                #Suggest only articles which are not in navigation yet
                suggestions.append({
                    'label': models.get_object_label(content_type, obj),
                    'value': obj.id,
                    'category': content_type.model_class()._meta.verbose_name.capitalize(),
                    'type': content_type.app_label + u'.' + content_type.model,
                })

    #Add suggestion for an empty node
    suggestions.append({
        'label': _(u"Node"),
        'value': 0,
        'category': _(u"Empty node"),
        'type': "",
    })
    response['suggestions'] = suggestions
    return response


def navnode_in_navigation(request, tree):
    """toogle the is_visible_flag of a navnode"""
    response = {}
    node_id = request.POST['node_id']
    node = models.NavNode.objects.get(tree=tree, id=node_id)  # get the node
    node.in_navigation = not node.in_navigation
    node.save()
    if node.in_navigation:
        response['message'] = _(u"The node is now visible.")
        response['label'] = _(u"Hide node in navigation")
        response['icon'] = "in_nav"
    else:
        response['message'] = _(u"The node is now hidden.")
        response['label'] = _(u"Show node in navigation")
        response['icon'] = "out_nav"
    return response


@login_required
def process_nav_edition(request, tree_id):
    """This handle ajax request sent by the tree component"""
    if request.method == 'POST' and request.is_ajax() and 'msg_id' in request.POST:
        try:
            #Get the current tree
            tree_class = get_navtree_class()
            tree = get_object_or_404(tree_class, id=tree_id)

            #check permissions
            perm_name = "{0}.change_{1}".format(tree_class._meta.app_label, tree_class._meta.module_name)
            if not request.user.has_perm(perm_name):
                raise PermissionDenied

            functions = (
                view_navnode, rename_navnode, remove_navnode, move_navnode,
                add_navnode, get_suggest_list, navnode_in_navigation,
            )
            supported_msg = {}
            #create a map between message name and handler
            #use the function name as message id
            for fct in functions:
                supported_msg[fct.__name__] = fct

            #Call the handler corresponding to the requested message
            response = supported_msg[request.POST['msg_id']](request, tree)

            #If no exception raise: Success
            response['status'] = 'success'
            response.setdefault('message', 'Ok')  # if no message defined in response, add something

        except KeyError, msg:
            response = {'status': 'error', 'message': u"Unsupported message : %s" % msg}
        except PermissionDenied:
            response = {'status': 'error', 'message': u"You are not allowed to add a node"}
        except ValidationError, ex:
            response = {'status': 'error', 'message': u' - '.join(ex.messages)}
        except Exception, msg:
            logger.exception("process_nav_edition")
            response = {'status': 'error', 'message': u"An error occured : %s" % msg}

        #return the result as json object
        return HttpResponse(json.dumps(response), content_type='application/json')
    raise Http404


@login_required
@popup_redirect
def change_newsletter_template(request, newsletter_id):
    """change newsletter template"""
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)

    if not request.user.has_perm('can_edit_newsletter', newsletter):
        raise PermissionDenied

    if request.method == "POST":
        form = forms.NewsletterTemplateForm(newsletter, request.user, request.POST)
        if form.is_valid():
            newsletter.template = form.cleaned_data['template']
            newsletter.save()
            return HttpResponseRedirect(newsletter.get_edit_url())
    else:
        form = forms.NewsletterTemplateForm(newsletter, request.user)

    return render_to_response(
        'coop_cms/popup_change_newsletter_template.html',
        {'form': form, 'newsletter': newsletter},
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def test_newsletter(request, newsletter_id):
    """test newsletter"""
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)

    if not request.user.has_perm('can_edit_newsletter', newsletter):
        raise PermissionDenied

    dests = settings.COOP_CMS_TEST_EMAILS

    if request.method == "POST":
        try:
            nb_sent = send_newsletter(newsletter, dests)

            messages.add_message(
                request, messages.SUCCESS,
                _(u"The test email has been sent to {0} addresses: {1}").format(nb_sent, u', '.join(dests))
            )
            return HttpResponseRedirect(newsletter.get_absolute_url())

        except Exception:
            messages.add_message(request, messages.ERROR, _(u"An error occured! Please contact your support."))
            logger.error(
                'Internal Server Error: {0}'.format(request.path),
                exc_info=sys.exc_info,
                extra={
                    'status_code': 500,
                    'request': request
                }
            )
            return HttpResponseRedirect(newsletter.get_absolute_url())

    return render_to_response(
        'coop_cms/popup_test_newsletter.html',
        {'newsletter': newsletter, 'dests': dests},
        context_instance=RequestContext(request)
    )


@login_required
@popup_redirect
def schedule_newsletter_sending(request, newsletter_id):
    """schedule sending"""
    newsletter = get_object_or_404(models.Newsletter, id=newsletter_id)
    instance = models.NewsletterSending(newsletter=newsletter)

    if request.method == "POST":
        form = forms.NewsletterSchedulingForm(request.POST, instance=instance)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(newsletter.get_edit_url())
    else:
        form = forms.NewsletterSchedulingForm(instance=instance, initial={'scheduling_dt': datetime.now()})

    return render_to_response(
        'coop_cms/popup_schedule_newsletter_sending.html',
        {'newsletter': newsletter, 'form': form},
        context_instance=RequestContext(request)
    )


@login_required
@popup_close
def add_fragment(request):
    """add a fragment to the current template"""
    
    content_type = ContentType.objects.get_for_model(models.Fragment)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
    if not request.user.has_perm(perm):
        raise PermissionDenied

    if request.method == "POST":
        form = forms.AddFragmentForm(request.POST)
        if form.is_valid():
            form.save()
            return None #popup_close decorator will close and refresh
    else:
        form = forms.AddFragmentForm()
        
    context_dict = {
        'form': form,
    }

    return render_to_response(
        'coop_cms/popup_add_fragment.html',
        context_dict,
        context_instance=RequestContext(request)
    )


@login_required
@popup_close
def edit_fragments(request):
    """edit fragments of the current template"""
    
    content_type = ContentType.objects.get_for_model(models.Fragment)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
    if not request.user.has_perm(perm):
        raise PermissionDenied
    
    edit_fragment_formset = modelformset_factory(models.Fragment, forms.EditFragmentForm, extra=0)

    if request.method == "POST":
        formset = edit_fragment_formset(request.POST, queryset=models.Fragment.objects.all())
        if formset.is_valid():
            formset.save()
            return None #popup_close decorator will close and refresh
    else:
        formset = edit_fragment_formset(queryset=models.Fragment.objects.all())
    
    context_dict = {
        'form': formset,
        'title': _(u"Edit fragments of this template?"),
    }

    return render_to_response(
        'coop_cms/popup_edit_fragments.html',
        context_dict,
        context_instance=RequestContext(request)
    )


def articles_category(request, slug):
    """view articles by category"""
    category = get_object_or_404(models.ArticleCategory, slug=slug, sites__id=settings.SITE_ID)
    
    if not request.user.has_perm('can_view_category', category):
        raise PermissionDenied()
    
    articles = category.get_articles_qs().filter(
        publication=models.BaseArticle.PUBLISHED
    ).order_by("-publication_date")
    
    if articles.count() == 0:
        raise Http404
    
    try:
        category_template = u"coop_cms/categories/{0}.html".format(category.slug)
        get_template(category_template)
    except TemplateDoesNotExist:
        category_template = "coop_cms/articles_category.html"
    
    return render_to_response(
        category_template,
        {'category': category, "articles": articles},
        context_instance=RequestContext(request)
    )


@csrf_exempt
def hide_accept_cookies_message(request):
    """force cookie warning message to be hidden"""
    if request.method == 'POST':
        request.session['hide_accept_cookie_message'] = True
        data = {"Ok": True}
        return HttpResponse(json.dumps(data), content_type="application/json")
    raise Http404


@csrf_exempt
def change_language(request):
    """change the language"""
    try:
        from localeurl import utils as localeurl_utils # pylint: disable=F0401
    except ImportError:
        raise Http404
    
    next_url = request.REQUEST.get('next', None)
    if not next_url:
        try:
            url = urlparse(request.META.get('HTTP_REFERER'))
            if url:
                next_url = url.path
        except Exception:
            pass
    
    if request.method == 'POST':
        lang_code = request.POST.get('language', None)
        after_change_url = request.POST.get('next_url_after_change_lang', None)
        if after_change_url:
            next_url = after_change_url
            
        if lang_code and check_for_language(lang_code):
            
            #path is the locale-independant url
            path = localeurl_utils.strip_path(next_url)[1]
            
            article_class = get_article_class()
            try:
                #get the translated slug of the current article
                #If switching from French to English and path is /fr/accueil/
                #The next should be : /en/home/
                
                #Get the article
                next_article = article_class.objects.get(slug=path.strip('/'))
                
            except article_class.DoesNotExist:
                next_article = None
            
            if hasattr(request, 'session'):
                request.session['django_language'] = lang_code
            #else:
                #response.set_cookie(settings.LANGUAGE_COOKIE_NAME, lang_code)
            activate(lang_code)
            
            if next_article:
                next_url = next_article.get_absolute_url()
            else:
                next_url = localeurl_utils.locale_path(path, lang_code)

    if not next_url:
        next_url = '/'
    
    return HttpResponseRedirect(next_url)


class ArticleView(EditableObjectView):
    """Article view for edition"""
    model = get_article_class()
    form_class = get_article_form()
    field_lookup = "slug"
    varname = "article"
    
    def get_object(self):
        """get object"""
        return get_article_or_404(slug=self.kwargs['slug'], sites=settings.SITE_ID)
    
    def can_access_object(self):
        """perms -> 404 if no perms"""
        if self.object.is_archived():
            return super(ArticleView, self).can_view_object()
        return True
    
    def handle_object_not_found(self):
        """go to alias if not found"""
        return redirect_if_alias(path=self.kwargs['slug'])
    
    def get_headlines(self):
        """headline"""
        return get_headlines(self.object)
    
    def get_context_data(self):
        """context"""
        context_data = super(ArticleView, self).get_context_data()
        context_data.update({
            'draft': self.object.publication == models.BaseArticle.DRAFT,
            'headlines': self.get_headlines(), 
            'ARTICLE_PUBLISHED': models.BaseArticle.PUBLISHED
        })
        return context_data
    
    def after_save(self, article):
        """after save"""
        if article.temp_logo:
            article.logo = article.temp_logo
            article.temp_logo = ''
            article.save()

    def get_template(self):
        """get template"""
        return get_article_template(self.object)


class EditArticleView(ArticleView):
    """editable version"""
    edit_mode = True


class NewsletterView(EditableObjectView):
    """newsletter view for edition"""
    model = models.Newsletter
    form_class = get_newsletter_form()
    field_lookup = "id"
    varname = "newsletter"

    def can_view_object(self):
        if self.object.is_public:
            return True
        return super(NewsletterView, self).can_edit_object()

    def get_context_data(self):
        """context"""
        context_data = super(NewsletterView, self).get_context_data()
        context_data.update({
            'title': self.object.subject,
        })
        return context_data

    def after_save(self, article):
        """after save"""
        pass

    def get_template(self):
        """get template"""
        return self.object.get_template_name()


class DebugErrorCodeView(TemplateView):
    """Debugging: view error page in debug"""

    def get_template_names(self):
        """template to view"""
        return ("{0}.html".format(self.kwargs["error_code"]),)


def csrf_failure(request, reason=""):
    """
    Custom view used when request fails CSRF protection.
    
    ENABLED by default by coop_cms unless you set COOP_CMS_DO_NOT_INSTALL_CSRF_FAILURE_VIEW=True in settings.py
    """
    
    logger.warn(u"csrf_failure, reason: {0}".format(reason))
    
    template = get_template('coop_cms/csrf_403.html')
    
    context = Context({
        'DEBUG': settings.DEBUG,
        'cookie_disabled': reason == REASON_NO_CSRF_COOKIE,
        'no_referer': reason == REASON_NO_REFERER,
    })

    html = template.render(RequestContext(request, context))
    
    return HttpResponseForbidden(html, content_type='text/html')



# -*- coding: utf-8 -*-

import os.path
import unicodedata

from bs4 import BeautifulSoup

from django import template
from django.conf import settings
from django.template import Context, RequestContext
from django.template.loader import get_template
from django.utils.text import slugify

from floppyforms import CheckboxInput

from coop_cms.models import ArticleCategory, Image
from coop_cms.settings import get_article_class
from coop_cms.shortcuts import get_article
from coop_cms.utils import dehtml as do_dehtml

register = template.Library()

################################################################################
class ArticleLinkNode(template.Node):
    """create an article and returns a link into template"""

    def __init__(self, title, lang):
        self.title = title
        self.lang = lang

    def render(self, context):
        """to html"""
        article_class = get_article_class()
        
        try:
            v = template.Variable(self.title)
            title = v.resolve(context)
        except template.VariableDoesNotExist:
            title = self.title.strip("'").strip('"')
        
        slug = slugify(title)
        try:
            if self.lang:
                article = get_article(slug, force_lang=self.lang)
            else:
                #If the language is not defined, we need to get it from the context
                #The get_language api doesn't work in templatetag
                request = context.get('request', None)
                lang = "en"
                if request:
                    lang = request.LANGUAGE_CODE
                elif hasattr(settings, 'LANGUAGES'):
                    lang = settings.LANGUAGES[0][0]
                elif hasattr(settings, 'LANGUAGE_CODE'):
                    lang = settings.LANGUAGE_CODE[:2]
                article = get_article(slug, current_lang=lang)
        except article_class.DoesNotExist:
            try:
                article = get_article(slug, all_langs=True)
            except article_class.DoesNotExist:
                article = article_class.objects.create(slug=slug, title=title)
        return article.get_absolute_url()


@register.tag
def article_link(parser, token):
    """template tag"""
    args = token.split_contents()
    title = args[1]
    lang = args[2] if len(args) > 2 else None
    return ArticleLinkNode(title, lang)


@register.filter
def dehtml(value):
    """html to text"""
    return do_dehtml(value)


@register.filter
def sp_rt_lb(value):
    """clean"""
    return value.replace("\n", " ").replace("\r", "")


################################################################################
class NewsletterFriendlyCssNode(template.Node):
    """css in tags attributes"""
    def __init__(self, nodelist_content, css):
        self.css = css
        self.nodelist_content = nodelist_content

    def render(self, context):
        """to html"""
        content = self.nodelist_content.render(context)
        if context.get('by_email', False):
            soup = BeautifulSoup(content)
            for tag, css in self.css.items():
                for html_tag in soup.select(tag):
                    html_tag["style"] = css
            content = soup.prettify(formatter="minimal")
        else:
            style = ""
            for tag, value in self.css.items():
                style += u"{0} {{ {1} }}\n".format(tag, value)
            content = u"<style>\n{0}</style>\n".format(style) + content
        return content

@register.tag
def nlf_css(parser, token):
    """Newsletter friendly CSS"""
    args = token.split_contents()
    css = {}
    for item in args[1:]:
        tag, value = item.split("=")
        tag, value = tag.strip('"'), value.strip('"')
        css[tag] = value
    nodelist = parser.parse(('end_nlf_css',))
    token = parser.next_token()
    return NewsletterFriendlyCssNode(nodelist, css)


@register.filter
def normalize_utf8_to_ascii(ustr):
    """utf to ascii"""
    try:
        return unicodedata.normalize('NFKD', ustr).encode('ascii','ignore')
    except TypeError:
        return ustr
    

@register.filter(name='is_checkbox')
def is_checkbox(field):
    """is checkbox"""
    field = getattr(field, 'field', field) # get the field attribute of the field or the field itself
    return field.widget.__class__.__name__ == CheckboxInput().__class__.__name__


@register.filter
def index(seq, index_val):
    """get index"""
    try:
        return seq[index_val]
    except IndexError:
        return None


################################################################################
class CoopCategoryNode(template.Node):
    """get category in template context"""
    def __init__(self, cat_slug, var_name):
        cat = cat_slug.strip("'").strip('"')
        self.cat_var, self.cat = None, None
        if cat_slug == cat:
            self.cat_var = template.Variable(cat)
        else:
            self.cat = cat
        self.var_name = var_name

    def render(self, context):
        """to html"""
        if self.cat_var:
            self.cat = self.cat_var.resolve(context)
        try:
            slug = slugify(self.cat)
            self.category = ArticleCategory.objects.get(slug=slug)
        except ArticleCategory.DoesNotExist:
            self.category = ArticleCategory.objects.create(name=self.cat)
        context.dicts[0][self.var_name] = self.category
        return ""

@register.tag
def coop_category(parser, token):
    """get category in template context"""
    args = token.split_contents()
    cat_slug = args[1]
    var_name = args[2]
    return CoopCategoryNode(cat_slug, var_name)


@register.filter
def basename(fullname):
    """os.path.basename"""
    return os.path.basename(fullname)


@register.filter
def get_parts(list_of_objs, number_of_parts):
    """slice"""
    nb_objs = len(list_of_objs)
    nb_by_part, extra_nb = nb_objs/number_of_parts, nb_objs % number_of_parts
    parts = []
    stop_index = 0
    for which_part in range(number_of_parts):
        start_index = 0 if (stop_index==0) else (stop_index)
        stop_index = start_index + nb_by_part + (1 if (which_part<extra_nb) else 0)
        parts.append(list_of_objs[start_index:stop_index])
    return parts


@register.filter
def get_part(list_of_objs, partionning):
    """slices"""
    which_part, number_of_parts = [int(x) for x in partionning.split("/")]
    parts = get_parts(list_of_objs, number_of_parts)
    return parts[which_part-1]
    

################################################################################
class ImageListNode(template.Node):
    """image list"""
    def __init__(self, filter_name, var_name):
        stripped_filter_name = filter_name.strip("'").strip('"')
        self.filter_var, self.filter_value = None, None
        if stripped_filter_name == filter_name:
            self.filter_var = template.Variable(filter_name)
        else:
            self.filter_value = stripped_filter_name
        self.var_name = var_name

    def render(self, context):
        if self.filter_var:
            self.filter_value = self.filter_var.resolve(context)
        images = Image.objects.filter(filters__name=self.filter_value).order_by("ordering", "-created")
        context.dicts[1][self.var_name] = images
        return ""

@register.tag
def coop_image_list(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        filter_name = args[1]
        as_name = args[2]
        var_name = args[3]
    except IndexError:
        raise Exception(u"coop_image_list: usage --> {% coop_image_list 'filter_name' as var_name %}")
    return ImageListNode(filter_name, var_name)


################################################################################
DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE = 'coop_cms/_accept_cookies_message.html'

class ShowAcceptCookieMessageNode(template.Node):
    """accept cookie message"""
    def __init__(self, template_name):
        self.template_name = template_name or DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE
        super(ShowAcceptCookieMessageNode, self).__init__()

    def render(self, context):
        """to html"""
        request = context.get('request', None)
        if not request.session.get('hide_accept_cookie_message', False):
            t = get_template(self.template_name)
            return t.render(RequestContext(request, {}))
        else:
            return ""


@register.tag
def show_accept_cookie_message(parser, token):
    """show accept cookie message"""
    args = token.split_contents()
    if len(args) > 1:
        template_name = args[1].strip('"').strip('"')
    else:
        template_name = ""
    return ShowAcceptCookieMessageNode(template_name)



# -*- coding: utf-8 -*-
"""login form with Email rather than Username"""

from django import forms
from django.contrib.auth import authenticate
from django.utils.translation import ugettext as _, ugettext_lazy as __


class EmailAuthForm(forms.Form):
    """Email form"""
    email = forms.EmailField(required=True, label=__(u"Email"))
    password = forms.CharField(label=__("Password"), widget=forms.PasswordInput)

    def __init__(self, request=None, *args, **kwargs):
        self.user_cache = None
        super(EmailAuthForm, self).__init__(*args, **kwargs)

    def _authenticate(self):
        """check authentication"""
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')

        error_messages = {
            'invalid_login': _("Please enter a correct %(email)s and password. "
                               "Note that both fields may be case-sensitive."),
        }

        if email and password:
            self.user_cache = authenticate(email=email, password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    error_messages['invalid_login'],
                    code='invalid_login',
                    params={'email': _(u"email")},
                )

    def get_user(self):
        """return the user"""
        return self.user_cache

    def clean(self):
        """clean data"""
        self._authenticate()
        return self.cleaned_data



# -*- coding: utf-8 -*-

from django.conf import settings
if 'localeurl' in settings.INSTALLED_APPS:
    from localeurl.models import patch_reverse
    patch_reverse()

import json
from unittest import skipIf

from django.contrib.auth.models import User, Permission, AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.test import TestCase
from django.test.client import RequestFactory

from coop_cms.models import Link, NavNode, BaseArticle
from coop_cms.settings import is_localized, is_multilang, get_article_class, get_navtree_class
from coop_cms.templatetags.coop_utils import get_part, get_parts
from coop_cms.tests import BaseTestCase


class CmsEditTagTest(BaseTestCase):

    def setUp(self):
        super(CmsEditTagTest, self).setUp()

        self.link1 = Link.objects.create(url='http://www.google.fr')
        self.tree = tree = get_navtree_class().objects.create()
        NavNode.objects.create(tree=tree, label=self.link1.url, content_object=self.link1, ordering=1, parent=None)

    def _log_as_editor(self):
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')

        ct = ContentType.objects.get_for_model(get_article_class())

        perm = 'change_{0}'.format(ct.model)
        can_edit_article = Permission.objects.get(content_type=ct, codename=perm)
        user.user_permissions.add(can_edit_article)

        perm = 'add_{0}'.format(ct.model)
        can_add_article = Permission.objects.get(content_type=ct, codename=perm)
        user.user_permissions.add(can_add_article)

        user.is_active = True
        user.save()

        return self.client.login(username='toto', password='toto')

    def _create_article(self):
        Article = get_article_class()
        article = Article.objects.create(
            title='test', content='<h1>Ceci est un test</h1>', publication=BaseArticle.PUBLISHED,
            template="test/nav_tag_in_edit_tag.html")
        NavNode.objects.create(tree=self.tree, label=article.title, content_object=article, ordering=1, parent=None)
        return article

    def test_view_navigation_inside_cms_edit_tag_visu(self):
        article = self._create_article()

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)

        self.assertContains(response, "Hello") #text in template
        self.assertContains(response, article.content)
        self.assertContains(response, self.link1.url)

    def test_view_navigation_inside_cms_edit_tag_edition(self):
        self._log_as_editor()
        article = self._create_article()

        response = self.client.get(article.get_edit_url(), follow=True)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, "Hello")
        self.assertContains(response, article.content)
        self.assertContains(response, self.link1.url)


class ArticleTemplateTagsTest(BaseTestCase):

    def _request(self):
        class DummyRequest:
            def __init__(self):
                self.LANGUAGE_CODE = settings.LANGUAGES[0][0]
        return DummyRequest()

    def test_create_article_link(self):
        tpl = Template('{% load coop_utils %}{% article_link "test" %}')
        html = tpl.render(Context({'request': self._request()}))

        Article = get_article_class()
        self.assertEqual(Article.objects.count(), 1)
        a = Article.objects.all()[0]
        self.assertEqual(a.slug, "test")

    def test_existing_article(self):
        Article = get_article_class()

        article = Article.objects.create(slug="test", title="Test")

        tpl = Template('{% load coop_utils %}{% article_link "test" %}')
        html = tpl.render(Context({'request': self._request()}))

        self.assertEqual(Article.objects.count(), 1)
        a = Article.objects.all()[0]
        self.assertEqual(a.slug, "test")

    @skipIf(len(settings.LANGUAGES)==0, "not languages")
    def test_article_link_language(self):

        lang = settings.LANGUAGES[0][0]

        tpl = Template('{% load coop_utils %}{% article_link "test" '+lang+' %}')
        html = tpl.render(Context({'request': self._request()}))

        Article = get_article_class()
        self.assertEqual(Article.objects.count(), 1)
        a = Article.objects.all()[0]
        self.assertEqual(a.slug, "test")

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_article_link_force_language(self):

        lang = settings.LANGUAGES[0][0]

        tpl = Template('{% load coop_utils %}{% article_link "test" '+lang+' %}')
        request = self._request()
        request.LANGUAGE_CODE = settings.LANGUAGES[1][0]
        html = tpl.render(Context({'request': request}))

        Article = get_article_class()
        self.assertEqual(Article.objects.count(), 1)
        a = Article.objects.all()[0]
        self.assertEqual(a.slug, "test")

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_article_existing_link_force_language(self):

        Article = get_article_class()

        lang = settings.LANGUAGES[0][0]

        article = Article.objects.create(slug="test", title="Test")

        request = self._request()
        lang = request.LANGUAGE_CODE = settings.LANGUAGES[1][0]

        setattr(article, "slug_"+lang, "test_"+lang)
        article.save()

        tpl = Template('{% load coop_utils %}{% article_link "test" '+lang+' %}')
        html = tpl.render(Context({'request': request}))

        self.assertEqual(Article.objects.count(), 1)
        a = Article.objects.all()[0]
        self.assertEqual(a.slug, "test")
        self.assertEqual(getattr(article, "slug_"+lang), "test_"+lang)

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_article_existing_link_force_default_language(self):

        Article = get_article_class()

        article = Article.objects.create(title="Test")

        request = self._request()
        def_lang = settings.LANGUAGES[0][0]
        cur_lang = request.LANGUAGE_CODE = settings.LANGUAGES[1][0]

        #activate(cur_lang)
        setattr(article, "slug_"+cur_lang, "test_"+cur_lang)
        article.save()

        count = Article.objects.count()

        tpl = Template('{% load coop_utils %}{% article_link "test" '+def_lang+' %}')
        html = tpl.render(Context({'request': request}))

        self.assertEqual(Article.objects.count(), count)
        a = Article.objects.get(id=article.id)
        self.assertEqual(a.slug, "test")
        self.assertEqual(getattr(a, "slug_"+cur_lang), "test_"+cur_lang)


class PartitionTemplateFilterTest(TestCase):
    
    def test_get_part_exact(self):
        objs = range(9)
        self.assertEqual([0, 1, 2], get_part(objs, "1/3"))
        self.assertEqual([3, 4, 5], get_part(objs, "2/3"))
        self.assertEqual([6, 7, 8], get_part(objs, "3/3"))
    
    def test_get_part_inexact(self):
        objs = range(10)
        self.assertEqual([0, 1, 2, 3], get_part(objs, "1/3"))
        self.assertEqual([4, 5, 6,], get_part(objs, "2/3"))
        self.assertEqual([7, 8, 9], get_part(objs, "3/3"))
        
    def test_get_part_empty(self):
        objs = []
        self.assertEqual([], get_part(objs, "1/3"))
        self.assertEqual([], get_part(objs, "2/3"))
        self.assertEqual([], get_part(objs, "3/3"))
    
    def test_get_part_less_than(self):
        objs = [0, 1]
        self.assertEqual([0], get_part(objs, "1/3"))
        self.assertEqual([1], get_part(objs, "2/3"))
        self.assertEqual([], get_part(objs, "3/3"))
        
    def test_get_parts_exact(self):
        objs = range(9)
        self.assertEqual([[0, 1, 2], [3, 4, 5], [6, 7, 8]], get_parts(objs, 3))
        
    def test_get_parts_inexact(self):
        objs = range(10)
        self.assertEqual([[0, 1, 2, 3], [4, 5, 6,], [7, 8, 9]], get_parts(objs, 3))

    def test_get_parts_empty(self):
        objs = []
        self.assertEqual([[], [], []], get_parts(objs, 3))
    
    def test_get_parts_less_than(self):
        objs = [0, 1]
        self.assertEqual([[0], [1], []], get_parts(objs, 3))


class AcceptCookieMessageTest(BaseTestCase):

    def test_get_hide_accept_cookies(self):

        url = reverse("coop_cms_hide_accept_cookies_message")
        response = self.client.get(url)

        self.assertEqual(404, response.status_code)

        self.assertEqual(self.client.session.get('hide_accept_cookie_message', None), None)


    def test_post_hide_accept_cookies(self):

        url = reverse("coop_cms_hide_accept_cookies_message")
        response = self.client.post(url)

        self.assertEqual(200, response.status_code)

        json_content = json.loads(response.content)
        self.assertEqual(json_content["Ok"], True)

        self.assertEqual(self.client.session.get('hide_accept_cookie_message'), True)


    def test_view_accept_cookies_message(self, ):
        tpl = Template('{% load coop_utils %}{% show_accept_cookie_message %}')

        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        request.session = {}

        html = tpl.render(Context({'request': request}))
        self.assertTrue(len(html) > 0)
        url = reverse("coop_cms_hide_accept_cookies_message")
        self.assertTrue(html.find(url) > 0)

    def test_view_accept_cookies_messages_hidden(self):
        tpl = Template('{% load coop_utils %}{% show_accept_cookie_message %}')

        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        request.session = {'hide_accept_cookie_message': True}

        html = tpl.render(Context({'request': request}))
        self.assertTrue(len(html) == 0)

    def test_view_accept_cookies_custom_template(self):
        tpl = Template('{% load coop_utils %}{% show_accept_cookie_message "test/_accept_cookies_message.html" %}')

        factory = RequestFactory()
        request = factory.get('/')
        request.user = AnonymousUser()
        request.session = {}

        html = tpl.render(Context({'request': request}))
        self.assertEqual(html, "Accept cookies")
